################################################################################
# YALTA FIXED-WIDTH ROLLING WINDOW MEBOOT INTERVAL ESTIMATIONS                 #
################################################################################

rolling.meboot <- function(inpT,wthW,prbC=0.8,bigJ=999){
  ##############################################################################
  # Written by A. Talha Yalta (November 2012). Please cite my relevant papers. #
  ##############################################################################
  # Computes fixed-width rolling window meboot HDR interval estimates using    #
  #                                    inpT : Data matrix (column 1 is y),     #
  #                                    wthW : Window width,                    #
  #                                    prbC : HDR probability coverage,        #
  #                                    bigJ : Number of replicates.            #
  #########################################                                    #
  timE <- proc.time()[3]                  # : Starting time                    #
  bigK <- ncol(inpT)                      # : Total number of variables        #
  estP <- matrix(NA,bigJ,bigK)            # : Meboot vectors for one window    #
  estA <- rep(NA,bigJ)                    # : AIC vector for one window        #
  numW <- nrow(inpT)-wthW+1               # : Total number of windows          #
  criT <- qt(1-((1-prbC)/2),df=wthW-bigK) # : Critical for asymptotic CIs.     #
  outM <- matrix(NA,numW,bigK*3)          # : Meboot estimates (low,pnt,high)  #
  outA <- matrix(NA,numW,bigK*3)          # : Asymptotic estms (low,pnt,high)  #
  outS <- matrix(NA,numW,11+bigK)         # : Summary stats (c1=R2, c2=adjR2,  #
  set.seed(12345)                         #   c3=DW, c4=AIC, c567=AICmeboot,   #
                                          #   c8,..=some more stats-see below) #
  tmp0 <- rnorm(10000000)                 # : Warming up the RNG.              #
    for(m in 1:numW){                     ######################################
      cat(paste("Computing window",m,"/",numW,"-",date(),"\n"))
      flush.console()
      ### Compute the asymptotic estimates
      olsP <- rep(NA,bigK)
      tmp1 <- ts(inpT[m:(m+wthW-1),])
      reg  <- lm(tmp1[,1] ~ tmp1[,2:ncol(tmp1)])
      for(k in 1:bigK){
        outA[m,k*3-2] <- summary(reg)$coef[k,1]-criT*summary(reg)$coef[k,2]
        outA[m,k*3-1] <- summary(reg)$coef[k,1]
        outA[m,k*3]   <- summary(reg)$coef[k,1]+criT*summary(reg)$coef[k,2]
        outS[m,1]     <- summary(reg)$r.sq
        outS[m,2]     <- summary(reg)$adj.r
        outS[m,3]     <- dwtest(reg)$stat
        outS[m,4]     <- AIC(reg)
      }
      ### Compute the meboot estimates
      ensM <- array(0, dim=c(wthW,bigJ,bigK)) 
      for(k in 1:bigK){
        ensM[,,k] <- meboot(x=inpT[m:(m+wthW-1),k], reps=bigJ)$ensemble
      }
      ensM <- aperm(ensM, c(1,3,2)) ### Transpose so that dim=wthW,bigK,bigJ)
      for(j in 1:bigJ){
        input  <- ts(ensM[,,j])
        reg <- lm(input[,1] ~ input[,2:ncol(input)])
        estP[j,1:bigK] <- summary(reg)$coef[1:bigK,1]
        estA[j] <- AIC(reg)
      }
      for(k in 1:bigK){
        tmp2 <- hdr(estP[,k], prob=prbC*100)        ############################
        outM[m,k*3-2] <- tmp2$hdr[1,1]              # betas HDR lower          #
        outM[m,k*3-1] <- tmp2$mode[1]               # betas HDR mode           #
        outM[m,k*3]   <- tmp2$hdr[1,ncol(tmp2$hdr)] # betas HDR upper          #
      }                                             #                          #
      tmp2 <- hdr(estA[],prob=prbC*100)             #                          #
      outS[m,5] <- tmp2$hdr[1,1]                    # AIC HDR lower            #
      outS[m,6] <- tmp2$mode[1]                     # AIC HDR mode             #
      outS[m,7] <- tmp2$hdr[1,ncol(tmp2$hdr)]       # AIC HDR upper            #
    }                                               ############################
    ############################################################################
    # Preparing below some more stats in outS. Cols8,9,10,11 show the averages #
    # of the first 4cols (R2,adjR2,DW,AIC averaged across windows). Cols after #
    # 11 show the interval lengths of all parameters averaged across windows.  #
    ############################################################################
    outS[1,8] <- mean(outS[,1])
    outS[1,9] <- mean(outS[,2])
    outS[1,10]<- mean(outS[,3])
    outS[1,11]<- mean(outS[,4])
    for(k in 1:bigK){
      outS[1,11+k] <- mean(outM[,k*3]-outM[,k*3-2])
    }

    cat(paste("Total time:",round((proc.time()[3]-timE)/60,3),"mins\n"))
    return(list(outM,outA,outS))
  }
  
  
  